/*
 * The "official" fmc-tdc API
 *
 * Copyright (C) 2012-2018 CERN (www.cern.ch)
 * Author: Tomasz Włostowski <tomasz.wlostowski@cern.ch>
 * Author: Alessandro Rubini <rubini@gnudd.com>
 *
 * SPDX-License-Identifier: LGPL-3.0-or-later
 */
#ifndef __FMCTDC_LIB_H__
#define __FMCTDC_LIB_H__

#ifdef __cplusplus
#pragma GCC diagnostic ignored "-Wwrite-strings"
extern "C" {
#endif

#include <stdint.h>

/**
 * Enumeration for all TDC channels
 */
enum fmctdc_channel {
	FMCTDC_CH_1 = 0,
	FMCTDC_CH_2,
	FMCTDC_CH_3,
	FMCTDC_CH_4,
	FMCTDC_CH_5,
	FMCTDC_CH_LAST = FMCTDC_CH_5,
	FMCTDC_NUM_CHANNELS = 5
};

/**
 * Enumeration of all buffer modes
 */
enum fmctdc_buffer_sw_mode {
	FMCTDC_BUF_SW_MODE_FIFO = 0, /**< FIFO policy: when buffer is full, new
					time-stamps will be dropped */
	FMCTDC_BUF_SW_MODE_CIRC, /**< circular buffer policy: when the buffer is
				    full, old time-stamps will be overwritten by
				    new ones */
};


/**
 * Enumeration for all possible status of a channel
 */
enum fmctdc_channel_status {
	FMCTDC_STATUS_DISABLE = 0, /**< The cannel is disable */
	FMCTDC_STATUS_ENABLE, /**< the channel is enable */
};

/**
 * Enumeration of all hardware buffer mode
 */
enum fmctdc_buffer_hw_mode {
	FMCTDC_BUF_HW_MODE_FIFO = 0,
	FMCTDC_BUF_HW_MODE_DMA,
};

/**
 * Enumeration for all possible time-stmap mode
 */
enum fmctdc_ts_mode {
	FMCTDC_TS_MODE_POST = 0, /**< after post-processing */
	FMCTDC_TS_MODE_RAW, /**< directly from ACAM chip. This should be used
			       ONLY when debugging low level issues */
};

/**
 * Enumeration of all possible White-Rabbit status
 */
enum fmctdc_wr_status {
	FMCTDC_WR_OFF = 0,
	FMCTDC_WR_ON_NOLINK,
	FMCTDC_WR_ON_SYNCING,
	FMCTDC_WR_ON,
};

/**
 * Opaque data type used as token. Do not try to access.
 */
struct fmctdc_board;


/**
 * FMC-TDC time-stamp descriptor
 */
struct fmctdc_time {
	uint64_t seconds; /**< TAI seconds. Note this is *not* an UTC time;
			     the counter does not support leap seconds. The
			     internal counter is also limited to 32 bits
			     (2038-error-prone). */
	uint32_t coarse; /**< number of ticks of 8ns since the beginning of
			    the last second*/
	uint32_t frac; /**< fractional part of an 8 ns tick, rescaled
			  to (0..4095) range - i.e. 0 = 0 ns, and
			  4095 = 7.999 ns. */
	uint32_t seq_id; /**< channel sequence number*/
	uint32_t debug;  /**< debug stuff, driver/firmware-specific */
};


/* Initialization */
extern int fmctdc_init(void);
extern void fmctdc_exit(void);
/* Utilities */
extern char *fmctdc_strerror(int err);
extern int fmctdc_temperature_get(struct fmctdc_board *userb,
				  float *temperature);
/* Open And Close Devices */
extern struct fmctdc_board *fmctdc_open(int offset, int dev_id);
extern struct fmctdc_board *fmctdc_open_by_lun(int lun);
extern int fmctdc_fileno_channel(struct fmctdc_board *b, unsigned int channel);
extern int fmctdc_close(struct fmctdc_board *);
/* Buffer Management */
extern int fmctdc_flush(struct fmctdc_board *userb, unsigned int channel);
/* Configuration */
extern int fmctdc_coalescing_timeout_set(struct fmctdc_board *userb,
					 unsigned int channel,
					 unsigned int timeout_ms);
extern int fmctdc_coalescing_timeout_get(struct fmctdc_board *userb,
					 unsigned int channel,
					 unsigned int *timeout_ms);
extern int fmctdc_status_set(struct fmctdc_board *userb,
			     unsigned int channel,
			     enum fmctdc_channel_status status);
extern int fmctdc_status_get(struct fmctdc_board *userb,
			      unsigned int channel,
			     enum fmctdc_channel_status *status);
extern int fmctdc_ts_mode_set(struct fmctdc_board *userb,
			      unsigned int channel,
			      enum fmctdc_ts_mode mode);
extern int fmctdc_ts_mode_get(struct fmctdc_board *userb,
			      unsigned int channel,
			      enum fmctdc_ts_mode *mode);
extern int fmctdc_offset_set(struct fmctdc_board *userb,
				unsigned int channel, int32_t offset);
extern int fmctdc_offset_get(struct fmctdc_board *userb,
				unsigned int channel, int32_t *offset);
extern int fmctdc_time_set(struct fmctdc_board *b, struct fmctdc_time *t);
extern int fmctdc_time_get(struct fmctdc_board *b, struct fmctdc_time *t);
extern int fmctdc_time_set_host(struct fmctdc_board *userb);
extern int fmctdc_wr_status_set(struct fmctdc_board *userb, int on);
extern int fmctdc_wr_status_get(struct fmctdc_board *userb,
				enum fmctdc_wr_status *status);
extern int fmctdc_termination_set(struct fmctdc_board *b,
				  unsigned int channel,
				  unsigned int enable);
extern int fmctdc_termination_get(struct fmctdc_board *b,
				  unsigned int channel,
				  unsigned int *enable);
extern int fmctdc_buffer_hw_mode_get(struct fmctdc_board *userb,
				     unsigned int channel,
				     enum fmctdc_buffer_hw_mode *mode);
extern int fmctdc_buffer_sw_mode_get(struct fmctdc_board *userb,
				     unsigned int channel,
				     enum fmctdc_buffer_sw_mode *mode);
extern int fmctdc_buffer_sw_mode_set(struct fmctdc_board *userb,
				     unsigned int channel,
				     enum fmctdc_buffer_sw_mode mode);

/* Acquire Time-Stamps */
extern int fmctdc_fread(struct fmctdc_board *b, unsigned int channel,
			struct fmctdc_time *t, int n);
extern int fmctdc_read(struct fmctdc_board *b, unsigned int channel,
		       struct fmctdc_time *t, int n, int flags);
/* Mathematical Operations */
extern void fmctdc_ts_sub(struct fmctdc_time *a, struct fmctdc_time *b);
extern void ft_ts_add(struct fmctdc_time *a, struct fmctdc_time *b);
extern int _fmctdc_tscmp(struct fmctdc_time *a, struct fmctdc_time *b);
/* Conversions */
extern uint64_t fmctdc_ts_approx_ns(struct fmctdc_time *a);
extern uint64_t fmctdc_ts_ps(struct fmctdc_time *a);
/* deprecated */
extern int fmctdc_get_buffer_len(struct fmctdc_board *userb,
				 unsigned int channel);
extern int fmctdc_set_buffer_len(struct fmctdc_board *userb,
				 unsigned int channel,
				 unsigned int length);
extern int fmctdc_reference_set(struct fmctdc_board *userb,
				unsigned int ch_target, int ch_reference);
extern int fmctdc_reference_get(struct fmctdc_board *userb,
				unsigned int ch_target);
extern int fmctdc_reference_clear(struct fmctdc_board *userb, int ch_target);
extern int fmctdc_set_time(struct fmctdc_board *b, struct fmctdc_time *t)
	__attribute__((deprecated));
extern int fmctdc_get_time(struct fmctdc_board *b, struct fmctdc_time *t)
	__attribute__((deprecated));
extern int fmctdc_set_host_time(struct fmctdc_board *b)
	__attribute__((deprecated));
extern int fmctdc_wr_mode(struct fmctdc_board *b, int on)
	__attribute__((deprecated));
extern int fmctdc_check_wr_mode(struct fmctdc_board *b)
	__attribute__((deprecated));
extern float fmctdc_read_temperature(struct fmctdc_board *b)
	__attribute__((deprecated));
extern int fmctdc_set_offset_user(struct fmctdc_board *userb,
				  unsigned int channel, int32_t offset)
	__attribute__((deprecated));
extern int fmctdc_get_offset_user(struct fmctdc_board *userb,
				  unsigned int channel, int32_t *offset)
	__attribute__((deprecated));
extern int fmctdc_set_termination(struct fmctdc_board *b, unsigned int channel,
				  int enable)
	__attribute__((deprecated));
extern int fmctdc_get_termination(struct fmctdc_board *b, unsigned int channel)
	__attribute__((deprecated));
extern int fmctdc_channel_status_set(struct fmctdc_board *userb,
				     unsigned int channel,
				     enum fmctdc_channel_status status)
	__attribute__((deprecated));
extern int fmctdc_channel_status_get(struct fmctdc_board *userb,
				     unsigned int channel)
	__attribute__((deprecated));
extern int fmctdc_channel_enable(struct fmctdc_board *userb,
				 unsigned int channel)
	__attribute__((deprecated));
extern int fmctdc_channel_disable(struct fmctdc_board *userb,
				  unsigned int channel)
	__attribute__((deprecated));
extern int fmctdc_get_buffer_mode(struct fmctdc_board *userb,
				  unsigned int channel)
		__attribute__((deprecated));
extern int fmctdc_set_buffer_mode(struct fmctdc_board *userb,
				  unsigned int channel,
				  enum fmctdc_buffer_sw_mode mode)
	__attribute__((deprecated));
/* other */

extern const char * const libfmctdc_version_s;
extern const char * const libfmctdc_zio_version_s;

#ifdef __cplusplus
}
#endif

#endif /* __FMCTDC_LIB_H__ */
